@@ -489,7 +489,7 @@ is.colGroup <- function(x) {
 #'   a locale is unsupported. When multiple locales are specified, the first
 #'   supported locale will be used.
 #'
-#'   See a list of [common BCP 47 language tags](https://docs.microsoft.com/en-us/openspecs/office_standards/ms-oe376/6c085406-a698-4e12-9d4d-c3b0ee3dbc4a)
+#'   See a list of [common BCP 47 language tags](https://learn.microsoft.com/en-us/openspecs/office_standards/ms-oe376/6c085406-a698-4e12-9d4d-c3b0ee3dbc4a)
 #'   for reference.
 #' @return A column format object that can be used to customize data formatting
 #'   in `colDef()`.
@@ -650,6 +650,9 @@ is.colFormat <- function(x) {
 }
 
 colType <- function(x) {
+  if (is.null(x)) {
+    return(NULL)
+  }
   if (is.numeric(x)) {
     return("numeric")
   }
